@@ -509,7 +509,9 @@ int ds_sds_dump_component_ref_as(const xmlNodePtr component_ref, struct ds_sds_s
 	// make a copy of xlink_href because ds_sds_dump_component_by_href modifies its second argument
 	char *xlink_href_copy = oscap_strdup(xlink_href);
 	int ret = ds_sds_dump_component_by_href(session, xlink_href, target_filename_dirname, relative_filepath, cref_id, &component_id);
-	oscap_htable_add(ds_sds_session_get_component_uris(session), cref_id, xlink_href_copy);
+	if (!oscap_htable_add(ds_sds_session_get_component_uris(session), cref_id, xlink_href_copy)) {
+		free(xlink_href_copy);
+	}
 
 	xmlFree(xlink_href);
 	xmlFree(cref_id);
